#include "../config.hpp"
#ifdef WIRELESS_SUPPORT


#include "esp_platform.hpp"
#include <Arduino.h>
#include <ArduinoOTA.h>
#include <Ticker.h>
#include <LITTLEFS.h>
#include <ESPAsyncWebServer.h>



/*===== Platform =====*/
    /*[ESP32]*/
    #if PLATFORM == ESP32
        #include <WiFi.h>
    #endif

    /*[ESP8266]*/
    #if PLATFORM == ESP8266
        #include <ESP8266WiFi.h>
    #endif



/*===== Object =====*/
AsyncWebServer server(80);
AsyncWebSocket ws("/ws");
Ticker temp_reporter;



/*===== Function =====*/

void handleWebSocketMessage(void *arg, uint8_t *data, size_t len) {
    AwsFrameInfo *info = (AwsFrameInfo*)arg;
    if (info->final && info->index == 0 && info->len == len && info->opcode == WS_TEXT) {
        data[len] = '\0';

        Serial.println((char*)data);
        String str_data = (String)(char*)data;
        String head = str_data.substring(0,3);
        String body = str_data.substring(3,str_data.length());
        #ifdef SERIAL_DEBUG
            Serial.println("head: "+ head + " body: " + body);
        #endif
        
        if(head == "h1t"){
        temp1_target = body.toDouble();
        }else if(head == "h2t"){
        temp2_target = body.toDouble();
        }
    }
}


void onEvent(AsyncWebSocket *server, AsyncWebSocketClient *client, AwsEventType type, void *arg, uint8_t *data, size_t len) {
    switch (type) {
        case WS_EVT_CONNECT:
            #ifdef SERIAL_DEBUG
                Serial.printf("WebSocket client #%u connected from %s\n", client->id(), client->remoteIP().toString().c_str());
            #endif
            temp1_target = HEAT_OFF;
            temp2_target = HEAT_OFF;
            break;
        case WS_EVT_DISCONNECT:
            #ifdef SERIAL_DEBUG
                Serial.printf("WebSocket client #%u disconnected\n", client->id());
            #endif
            temp1_target = HEAT_OFF;
            temp2_target = HEAT_OFF;
            break;
        case WS_EVT_DATA:
            handleWebSocketMessage(arg, data, len);
            break;
        case WS_EVT_PONG:
        case WS_EVT_ERROR:
            temp1_target = HEAT_OFF;
            temp2_target = HEAT_OFF;
            break;
    }
}


void web_report_handler(){
    ws.textAll("h1c" + String(temp1_current));
    ws.textAll("h2c" + String(temp2_current));
    ws.textAll("h1t" + String(temp1_target));
    ws.textAll("h2t" + String(temp2_target));
    #ifdef SERIAL_DEBUG
        Serial.println("report temp" + String(" h1c: ") + String(temp1_current) + " h2c: " + String(temp2_current) + " h1t: " + String(temp1_target) + " h2t: " + String(temp2_target));
    #endif
}


void wireless_setup(){
    /*[LittleFS Init]*/
    if(!LittleFS.begin()){
        #ifdef SERIAL_DEBUG
            Serial.println("fail init fs");
        #endif
        return;
    }


    /*[Show All Files]*/
    #ifdef SERIAL_DEBUG
        Dir root = LittleFS.openDir("/");
        while (root.next()) {
            File file = root.openFile("r");
            Serial.print(" FILE: ");
            Serial.print(root.fileName());
            Serial.print(" SIZE: ");
            Serial.println(file.size());
            file.close();
        }
    #endif


    /*[Wifi Init]*/
    WiFi.softAP(AP_SSID, AP_PASSWD, AP_CHANNEL, 0, AP_MAX_CLIENT);


    /*[Web Init]*/
    ws.onEvent(onEvent);
    server.addHandler(&ws);
    server.onNotFound([](AsyncWebServerRequest *request){ request->redirect("/"); }); //captive portal redirect
    server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){ request->send(LittleFS, "/main.html", "text/html"); }); //main html
    server.on("/main.css", HTTP_GET, [](AsyncWebServerRequest *request){ request->send(LittleFS, "/main.css", "text/css"); }); //main css

    server.begin();


    /*[OTA Init]*/
    ArduinoOTA.begin();


    /*[Ticker Init]*/
    temp_reporter.attach(0.5, web_report_handler);
}


void wireless_handler(){
    ws.cleanupClients();
    ArduinoOTA.handle();
}



#endif